#!/usr/bin/perl -w
#### Monitoring  ESXi (VMware Infrastructure (VI) API.)
#### Dmitry Maksimov 12.01.2010
####### Config ##############

$ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;

my $IP;
if (!defined($ARGV[0])) {$IP="192.168.10.9"}
  else {$IP=$ARGV[0]}
my $password;
if (!defined($ARGV[2])) {$password='12345678'}
  else {$password=$ARGV[2]}
my $username;
if (!defined($ARGV[1])) {$username="root"}
  else {$username=$ARGV[1]}
my $hostname;
if (!defined($ARGV[3])) {$hostname="ESXi"}
  else {$hostname=$ARGV[3]}
my $service_url = "https://".$IP."/sdk";
my $statusfile="/tmp/vmware_api_$IP.txt";
my $dsfile="/tmp/vmware_ds_$IP.txt";
my $vmfile="/tmp/vmware_vm_$IP.txt";
my $netfile="/tmp/vmware_net_$IP.txt";
my $diskfile="/tmp/vmware_disk_$IP.txt";

# key =>data
my %API=(
	"HOST.CPU.Total"	=>0
	,"HOST.CPU.Used"	=>0
	,"HOST.MEM.Size"	=>0
	,"HOST.MEM.Usage"	=>0
	,"HOST.MaintenanceMode"=>0
);

my %ARRAY=();

# error_code
# 0 - Ok
# 1 - error with perl API
# 2 - error with status file. Can't write status file.
# 3 - bad username or password
####### End Config ##########
use strict;
use warnings;
use lib "/usr/local/share/examples/vmware-vsphere-cli/";

use VMware::VIRuntime;
use VMware::VICommon;
use VMware::VIExt;
use AppUtil::VMUtil;
my $i;
my $code;
my $error_code=0;

#debugging - autoflush stdout
$| = 1;

if ($#ARGV < 0) {
	$error_code=1;
	print "Argument $error_code\n";
        exit 0;
}

eval {
# Login to Virtual Center service:
my $retval =Vim::login(service_url => $service_url, user_name => $username, password => $password);
      };
      if ($@) 
	{
          if ($@ =~ /incorrect user name or password/) {
            $error_code=3;            
         } else {
           $error_code=1;
         }
       }
if  ($error_code>0) { print "Login $error_code\n"; exit 0; }
my $host = Vim::find_entity_view(
view_type => 'HostSystem',
properties => [ 'summary' ],
);

my %host_status = (
   'gray' => 'The status is unknown',
   'green' => 'The entity is OK',
   'red' => 'The entity definitely has a problem',
   'yellow' => 'The entity might have a problem',
);

my ($cpuMhz, $numCores);
my $summary = $host->get_property("summary");

$cpuMhz = $summary->hardware->cpuMhz;
$numCores = $summary->hardware->numCpuCores;
$API{"HOST.CPU.Total"} = $cpuMhz * $numCores * 1000000; #in hz
$API{"HOST.CPU.Used"} = $summary->quickStats->overallCpuUsage * 1000000; #in hz
$API{"HOST.MEM.Size"} = $summary->hardware->memorySize; # in Byte
$API{"HOST.MEM.Usage"} = $summary->quickStats->overallMemoryUsage * 1048576 ; # in Byte
$API{"HOST.Status"}=$summary->overallStatus->val;

my $host_view = VIExt::get_host_view(1);
my $datastoreRefs = $host_view->datastore;
my $mounts = $host_view->config->fileSystemVolume->mountInfo;
my @datastores = ();
my @DSnames = ();
my @Disknames = ();
foreach (@$datastoreRefs) {
 my $datastore = Vim::get_view(mo_ref => $_);
 push (@datastores, $datastore);
 }
                  
my $first = 1;
if (-e $dsfile) {unlink($dsfile);}
open FILE,">$dsfile";
print FILE "{\n";
print FILE "\t\"data\":[\n\n";

my $datastorecounter = 0;
foreach (@datastores)
 {
 my $type = "";
 my $name = "";
 my $UUID = "";
 my $capacity = "";
 my $free = "??";
 my $partition = "";
 my $partitions = "";
 my $diskName;
 my $vol;
 my $uuid;
 my $info;
 my $extents;
 my $numExtents = 0;

 $free = $_->summary->freeSpace;
 $name = $_->summary->name;
 push (@DSnames,$name);
 $capacity = $_->summary->capacity;
 $extents = $_->info->vmfs->extent;
 foreach (@$extents){
    $diskName =  $_->diskName;
    }
 $name=~s/ /_/g;
 $name=~s/"//g;
 $name=~s/:/_/g;
 $API{"DS.".$name.".Capacity"}=$capacity; # in Byte
 $API{"DS.".$name.".Free"}=$free; # in Byte
 $API{"DS.".$name.".DiskName"}=$diskName; 
 push (@Disknames,$diskName);
 print FILE "\t,\n" if not $first;
 $first = 0;
 print FILE "\t{\n";
 print FILE "\t\t\"{#DSNAME}\":\"$name\"\n";
 print FILE "\t}\n";
 }
print FILE "\n\t]\n";
print FILE "}\n";
close(FILE);

$first = 1;
if (-e $diskfile) {unlink($diskfile);}
open FILE,">$diskfile";
print FILE "{\n";
print FILE "\t\"data\":[\n\n";
foreach (@Disknames)
    {
     print FILE "\t,\n" if not $first;
     $first = 0;
     print FILE "\t{\n";    
     print FILE "\t\t\"{#DISKNAME}\":\"$_\"\n";
     print FILE "\t}\n";     
    }
print FILE "\n\t]\n";
print FILE "}\n";
close(FILE);

if (defined($host_view->{runtime})) 
 {
 if ($host_view->{runtime}->{inMaintenanceMode}) 
   {
   $API{MaintenanceMode}=1; # in MaintenanceMode
   }else
	{
	$API{MaintenanceMode}=0; # not in MaintenanceMode
	}
 }

#Network Adapter
if (-e $netfile) {unlink($netfile);}
open FILE,">$netfile";
print FILE "{\n";
print FILE "\t\"data\":[\n\n";
$first = 1;

unless (defined($host_view->configManager->networkSystem)) {
   VIExt::fail("Error: network system not found.\n");
}
my $netsys = Vim::get_view (mo_ref => $host_view->configManager->networkSystem);
my $pnics = $netsys->networkInfo->pnic;
foreach my $pnic (@$pnics) 
{
      my ($state, $speed, $duplex);
      my $name="";
      my $ls = $pnic->linkSpeed;
      my $mtu_val = "";
      my $pci_device = find_pci_device($pnic->pci);

      if (defined($ls)) 
	{
         
	 $state = "1";
         $speed = $ls->speedMb * 1048576;
         $duplex = $ls->duplex ? "1" : "0";
       } else {
         $state = "0";
         $speed = "0";
         $duplex = "0";
      	      }

#      my $description = "";
#      eval {
#         $description = $pci_device->vendorName . " " . $pci_device->deviceName;
#           };
      $name=$pnic->device;
      $name=~s/ /_/g;
      $name=~s/"//g;
      $name=~s/:/_/g;
      $API{"NET.".$name.".state"}=$state;    # Interface 1-UP 0-Down
      $API{"NET.".$name.".speed"}=$speed;    # Speed bps
      $API{"NET.".$name.".duplex"}=$duplex;  # 1- Full 0-Half
      
      print FILE "\t,\n" if not $first;
      $first = 0;
      print FILE "\t{\n";
      print FILE "\t\t\"{#NETNAME}\":\"$name\"\n";
      print FILE "\t}\n";
}
print FILE "\n\t]\n";
print FILE "}\n";
close(FILE);
if (-e $vmfile) {unlink($vmfile);}
open FILE, ">$vmfile";
$first = 1;
 
print FILE "{\n";
print FILE "\t\"data\":[\n\n";
 
my $vm_views = Vim::find_entity_views(view_type => "VirtualMachine");
foreach my $vm_view (@$vm_views) 
	{
	#my $vm_view=$_;
	my $VMname;
	$VMname = $vm_view->config->name;
	if ($vm_view->runtime->powerState->val eq 'poweredOn') 
		{
		#print $VMname; 
		my $VMStats = $vm_view->summary->quickStats;
		$API{"VM.".$VMname.".Cpu"}=$VMStats->overallCpuUsage * 1000000; #in Hz
		$API{"VM.".$VMname.".HMem"}=$VMStats->hostMemoryUsage * 1048576; #in Bytes
		$API{"VM.".$VMname.".GMem"}=$VMStats->guestMemoryUsage * 1048576; #in Bytes
		print FILE "\t,\n" if not $first;
	        $first = 0;
	        print FILE "\t{\n";
	        print FILE "\t\t\"{#VMNAME}\":\"$VMname\",\n";
                print FILE "\t}\n";
		}
	else
		{
		$API{"VM.".$VMname.".Cpu"}=0; 
		$API{"VM.".$VMname.".HMem"}=0;
		$API{"VM.".$VMname.".GMem"}=0;
		}
	}
print FILE "\n\t]\n";
print FILE "}\n";
close(FILE);
my $all_counters;
retrieve_performance(countertype=>"sys");
retrieve_performance(countertype=>"mem");
retrieve_performance(countertype=>"net");
retrieve_performance(countertype=>"disk");

if (-e $statusfile) {unlink($statusfile);}

foreach $i (keys %API)
 {
 $code=Write_Status(FILE=>$statusfile,NAME=>"$i",STATUS=>"$API{$i}");
 if ($code==1) {$error_code=2};
 }
print "Status file $error_code\n";

#send value
`zabbix_sender -z zabbix -s $hostname -k GetCounterESX -o $error_code`;


sub Write_Status
{
 my %args=(
    FILE=>"",
    NAME=>"",
    STATUS=>"",
    @_);
local *FILE;
my $i;
my $error=0;
my %hash=();
if (!(-f $args{FILE}))
                {
                 open(FILE,">$args{FILE}")|| ($error=1);
                 close(FILE);
                }
%hash=Read_Status(FILE=>"$args{FILE}");
$hash{$args{NAME}}=$args{STATUS};
open(FILE,">$args{FILE}")|| ($error=1);
foreach  $i (keys(%hash))
 {
 print FILE "$i:\t$hash{$i}\n";
 }
close(FILE);
return $error;
}

sub Read_Status
{
my %args=(
    FILE=>"",
    @_);
local *FILE;
my ($name,$status);
my %hash=();
if (!(-f $args{FILE}))
                {
                 open(FILE,">$args{FILE}");
                 close(FILE);
                }
open(FILE,"$args{FILE}");
while(<FILE>)
 {
 chop;
 ($name,$status)=split(/\t+/,$_,2);
 $name=~s/://g;
 $hash{$name}=$status;
 }
close(FILE);
return %hash;
}

sub find_pci_device 
{
   my ($pci_id) = @_;

   my $pci_devices = $host_view->hardware->pciDevice;
   foreach my $pci_dev (@$pci_devices) {
      if (defined($pci_dev->{id}) && $pci_dev->{id} eq $pci_id) {
         return $pci_dev;
      }
   }
}


sub retrieve_performance {
   my %args=(countertype=>"net",
          @_);
   my $host = Vim::find_entity_view(view_type => "HostSystem");
   if (!defined($host)) {
      Util::trace(0,"Host ".$IP." not found.\n");
      return;
   }
  
   my $perfmgr_view = Vim::get_view(mo_ref => Vim::get_service_content()->perfManager);
  
   my @perf_metric_ids = get_perf_metric_ids(perfmgr_view=>$perfmgr_view,
                                             host => $host,
                                             type => $args{countertype});

   my $perf_query_spec;
      my $intervals = get_available_intervals(perfmgr_view => $perfmgr_view,
                                              host => $host);
      $perf_query_spec = PerfQuerySpec->new(entity => $host,
                                            metricId => @perf_metric_ids,
#                                            'format' => 'csv',
					    'format' => 'normal',
                                            intervalId => shift @$intervals,
                                            maxSample => 1);

   my $perf_data;
   eval {
       $perf_data = $perfmgr_view->QueryPerf( querySpec => $perf_query_spec);
   };
   if ($@) {
      if (ref($@) eq 'SoapFault') {
         if (ref($@->detail) eq 'InvalidArgument') {
            Util::trace(0,"Specified parameters are not correct");
         }
      }
      return;
   }
   if (! @$perf_data) 
   {
      Util::trace(0,"Either Performance data not available for requested period "
                    ."or instance is invalid\n");
      $intervals = get_available_intervals(perfmgr_view=>$perfmgr_view,
                                           host => $host);
      Util::trace(0,"\nAvailable Intervals\n");
      foreach(@$intervals) 
      {
         Util::trace(0,"Interval " . $_ . "\n");
      }
      return;
   }
   foreach (@$perf_data) 
    {
#      print "Performance data for: " . $host->name . "\n";
#      my $time_stamps = $_->sampleInfoCSV;
      my $values = $_->value;
      my $diskcounter = 0;
      foreach (@$values)
       {
#         print_counter_info($_->id->counterId, $_->id->instance);
#	 print "test ".$_->id->instance."\n";
#        print("Sample info : " . $time_stamps);
         my $test=$_->value;
         my @Value=@$test;
#         print("Value: " . $Value[0] . "\n");
         my $name=$_->id->counterId;
         my $counter = $all_counters->{$name};
         $name=$counter->nameInfo->label;
         if ($args{countertype} eq "mem") {$name=$counter->nameInfo->key}
	 if ($args{countertype} eq "net" || $args{countertype} eq "disk") {$name=$_->id ->instance}
         $name=~s/ /_/g;
         $name=~s/"//g;
         $name=~s/:/_/g;
         if ($args{countertype} eq "sys") 
	  {
	  $API{$name}=$Value[0];  # Uptime in sec
	  }
         if ($args{countertype} eq "mem")
          {
          $API{"HOST.MEM.".$name}=$Value[0] * 1024; # in Byte
          }
	 if ($args{countertype} eq "net")
          {
	  my $type=$counter->rollupType->val;
	  my $desc=$counter->nameInfo->label;
          my $units=$counter->unitInfo->label;
	  $desc=~s/ /_/g;
          $desc=~s/"//g;
          $desc=~s/:/_/g;
	  if ($name ne "" && $desc ne "" )
           {
	   if ($units eq "KBps")
	    {
	     if (lc($desc) eq "data_transmit_rate" ) {$desc="Network_Data_Transmit_Rate";}
	     if (lc($desc) eq "data_receive_rate" ) {$desc="Network_Data_Receive_Rate";}
            $API{$name.".".$desc.".".$type}=$Value[0] * 8192; # in bps
	    }else{
		 $API{$name.".".$desc.".".$type}=$Value[0]; # Number 
		 }
	   }
          }
	if ($args{countertype} eq "disk" && $name ne "")
          {
          my $type=$counter->rollupType->val;
          my $desc=$counter->nameInfo->label;
          my $units=$counter->unitInfo->label;
	  #print "disk $name type $type desc $desc\n";
	  my $count = 0;

#	  for(;$count < $datastorecounter + 1; $count++)
	  foreach (@DSnames)
	    {
	    if (defined($API{"DS.".$_.".DiskName"}))
		{
	    if ($API{"DS.".$_.".DiskName"} eq $name)
		{
#		$API{"Disk.".$count.".Name"}=$name;
		
		$desc=~s/ /_/g;
	        $desc=~s/"//g;
        	$desc=~s/:/_/g;
        	
        	if ($name ne "" && $desc ne "" )
        	    {
        	    if ($units eq "KBps")
        		{
        		$API{"Disk.".$name.".".$desc.".".$type}=$Value[0] * 1024; # in Bps
        		}
        	    else{
                	$API{"Disk.".$name.".".$desc.".".$type}=$Value[0]; # Number or Millisecond
                	}
        	    }
		}
		}
	    }
	  
	  
	  
          }

       }
   }
}

sub print_counter_info {
   my ($counter_id, $instance) = @_;
   my $counter = $all_counters->{$counter_id};
   print("\nCounter: " . $counter->nameInfo->label);
   if (defined $instance) {
      print("Instance : " . $instance);
   }
   print("Description: " . $counter->nameInfo->summary);
#   print("Units: " . $counter->unitInfo->label);
    print("Type: " .$counter->rollupType->val)
}

sub get_perf_metric_ids {
   my %args = @_;
   my $perfmgr_view = $args{perfmgr_view};
   my $entity = $args{host};
   my $type = $args{type};

   my $counters;
   my @filtered_list;
   my $perfCounterInfo = $perfmgr_view->perfCounter;
   my $availmetricid = $perfmgr_view->QueryAvailablePerfMetric(entity => $entity);

   foreach (@$perfCounterInfo) {
      my $key = $_->key;
      $all_counters->{ $key } = $_;
      my $group_info = $_->groupInfo;
      if ($group_info->key eq $type) {
         $counters->{ $key } = $_;
      }
   }

   foreach (@$availmetricid) {
	 my $metric;
      if (exists $counters->{$_->counterId}) {
         #push @filtered_list, $_;
	 if ($type eq "disk" || $type eq "net")
	  {
          $metric = PerfMetricId->new (counterId => $_->counterId,
                                          instance => '*');
	  } else {
		 $metric = PerfMetricId->new (counterId => $_->counterId,
                                          instance => '');
		 }
         push @filtered_list, $metric;
      }
   }
   return \@filtered_list;
}

sub get_available_intervals {
   my %args = @_;
   my $perfmgr_view = $args{perfmgr_view};
   my $entity = $args{host};

   my $historical_intervals = $perfmgr_view->historicalInterval;
   my $provider_summary = $perfmgr_view->QueryPerfProviderSummary(entity => $entity);
   my @intervals;
   if ($provider_summary->refreshRate) {
      push @intervals, $provider_summary->refreshRate;
   }
   foreach (@$historical_intervals) {
      push @intervals, $_->samplingPeriod;
   }
   return \@intervals;
}
