#!/usr/bin/perl -w
#
# Copyright (c) 2007 VMware, Inc.  All rights reserved.
$ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;

use strict;
use warnings;

use FindBin;
use lib "/usr/local/share/examples/vmware-vsphere-cli/";

use VMware::VIRuntime;
use VMware::VICommon;
use VMware::VIExt;
use AppUtil::VMUtil;


$Util::script_version = "1.0";

my $logKey = "hostd";
my $sleepTime;
my $lastNLines;
my $hostname;
my $start;
my $lines=0;

my $IP;
if (!defined($ARGV[0])) {$IP="192.168.10.9"}
  else {$IP=$ARGV[0]}
my $password;
if (!defined($ARGV[2])) {$password='12345678'}
    else {$password=$ARGV[2]}
my $username;
if (!defined($ARGV[1])) {$username="root"}
    else {$username=$ARGV[1]}

my $service_url = "https://".$IP."/sdk";

my $logfile = "/tmp/vmware_log_$IP.txt";
my $statlogfile = "/tmp/vmware_statlog_$IP.txt";

my $oldpos;
 if (-e $statlogfile) {
 $oldpos = get_pos($statlogfile);
}
else {
    $oldpos = 0;
}

my $i;
my $code;
my $error_code=0;

if ($#ARGV < 0) {
    $error_code=1;
    print "Error argument $error_code\n";
    exit 0;
}
                        
eval {
    # Login to Virtual Center service:
    my $retval =Vim::login(service_url => $service_url, user_name => $username, password => $password);
};
if ($@)
    {
    if ($@ =~ /incorrect user name or password/) {
	$error_code=3;
    } else {
	$error_code=1;
    }
}

if  ($error_code>0) { print "Error login $error_code\n"; exit 0; }


my $host = Vim::find_entity_view(
view_type => 'HostSystem',
properties => [ 'summary' ],
);

my $host_view = VIExt::get_host_view(1);
my $instance_type = Vim::get_service_content()->about->apiType;



my $diagmgr_view;
get_diagnosticManager();

sub get_diagnosticManager {
   $diagmgr_view =
   Vim::get_view(mo_ref => Vim::get_service_content()->diagnosticManager);
   unless ($diagmgr_view) {
      Util::trace(0, "Error Diagnostic Manager not found.\n");
   }
   if(defined $diagmgr_view){
         getLogData($logKey);
  }
}

sub getLogData {
   my $logData;
   my $lineEnd;
   my ($logKey) = @_;
      eval {           
         if ($instance_type eq 'VirtualCenter') {
            $logData = $diagmgr_view->BrowseDiagnosticLog(key => $logKey,
                                                          host => $host,
                                                          start => "999999999");
            $lineEnd = $logData->lineEnd;
            $start = $lineEnd - $lastNLines;
            $logData = readLogFile($diagmgr_view, $logKey, $start);
         }
         else {
            $logData = $diagmgr_view->BrowseDiagnosticLog(key => $logKey,
                                                          start => $oldpos);
            $lineEnd = $logData->lineEnd;
	    #was rotate log or clearlog
            if ($lineEnd < $oldpos) {
        	$oldpos = 0;
            }
            save_pos($lineEnd, $statlogfile);
            $logData = readLogFile($diagmgr_view, $logKey, $oldpos + 1);
         }
      };
      if ($@) {
         if (ref($@) eq 'SoapFault') {
            if (ref($@->detail) eq 'CannotAccessFile') {
               Util::trace(0,"\nkey refers to a file that cannot".
                             " be accessed at the present time.\n");
            }
            elsif (ref($@->detail) eq 'InvalidArgument') {
               Util::trace(0,"\nkey refers to a nonexistent log file".
                             " or the log file is not of type plain for " .
                             "host $hostname.\n");
            }
            elsif (ref($@->detail) eq 'RuntimeFault') {
                Util::trace(0,"\nRuntime fault.\n");
            }
            else {
                Util::trace(0, "Error: "  . $@ . " ");
            }
         }
         else {
             Util::trace(0, "Error: "  . $@ . " ");
         }
      }
}

sub readLogFile {
   my $logData;
   my ($diagMgr, $logKey, $start) = @_;
   eval {
      if ($instance_type eq 'VirtualCenter') {
         $logData = $diagmgr_view->BrowseDiagnosticLog(key => $logKey,
                                               host => $host,
                                               start => $start,
                                               );
      }
      else {
          $logData = $diagmgr_view->BrowseDiagnosticLog(key => $logKey,
                                               start => $start,
                                               );
      }
   };
   if ($@) {
      if (ref($@) eq 'SoapFault') {
         if (ref($@->detail) eq 'CannotAccessFile') {
            Util::trace(0,"\nkey refers to a file that cannot".
                                         " be accessed at the present time.\n");
         }
         elsif (ref($@->detail) eq 'InvalidArgument') {
            Util::trace(0,"\nkey refers to a nonexistent log file".
                                    " or the log file is not of type plain.\n");
         }
         elsif (ref($@->detail) eq 'RuntimeFault') {
            Util::trace(0,"\nRuntime fault.\n");
         }
         else {
            Util::trace(0, "Error: "  . $@ . " ");
         }
      }
      else {
         Util::trace(0, "Error: "  . $@ . " ");
      }
   }
   if ($logData->lineStart != 0) {
         foreach my $line (@{$logData->lineText}) {
            if ($line !~ m/verbose/ & $line !~ m/-->/ ){
                Util::trace(0, "" .$line."\n");
            }
         }
    }
   return $logData;
}


sub save_pos {
    my $pos = $_[0];
    my $file = $_[1];
    open(POS, ">$file") || die "Error save pos file";
    print POS "$pos";
    close(POS);
}

sub get_pos {
    my $file = $_[0];
    my $pos = 0;
    open(POS, "<$file") || die "Error open pos file";;
    $pos=(<POS>);
    if ($pos !~ /^\d+/) {
	$pos=0;
	}
    close(POS);
    return $pos;
} 



